unit LazIpHlpApi;

{$mode objfpc}{$H+}

interface

uses
  Windows, SysUtils, synaip;

const
  MAX_ADAPTER_DESCRIPTION_LENGTH = 128;
  MAX_ADAPTER_NAME_LENGTH = 256;
  MAX_ADAPTER_ADDRESS_LENGTH = 8;

type

  TAdaptersInfo = record
    Description: String;
    IpAddress: String;  // or array of String; ?
    MacAddress: String;
    SubNetMask: String;
    Gateway: String; // or array of String; ?
    DHCPServer: String;
    Broadcast: String;
    DNSServer: String; // or array of String; ?
  end;

  TAdapterInfosA = array of TAdaptersInfo;


  PIP_ADDRESS_STRING = ^IP_ADDRESS_STRING;
  IP_ADDRESS_STRING = packed record
    ToString: array [1..16] of Char;
  end;

  IP_MASK_STRING = IP_ADDRESS_STRING;

  PIP_ADDR_STRING = ^IP_ADDR_STRING;
  IP_ADDR_STRING = packed record
    Next: PIP_ADDR_STRING;
    IpAddress: IP_ADDRESS_STRING;
    IpMask: IP_MASK_STRING;
    Context: DWORD;
  end;

  time_t = comp;

  PIP_ADAPTER_INFO = ^IP_ADAPTER_INFO;
  IP_ADAPTER_INFO = packed record
    Next: PIP_ADAPTER_INFO;
    ComboIndex: DWORD;
    AdapterName: array [1..MAX_ADAPTER_NAME_LENGTH+4] of Char ;
    Description: array [1..MAX_ADAPTER_DESCRIPTION_LENGTH+4] of Char;
    AddressLength: UINT;
    Address: array [1..MAX_ADAPTER_ADDRESS_LENGTH] of Byte;
    Index: DWORD;
    dwType: UINT;
    DhcpEnabled: UINT;
    CurrentIpAddress: PIP_ADDR_STRING;
    IpAddressList: IP_ADDR_STRING;
    GatewayList: IP_ADDR_STRING;
    DhcpServer: IP_ADDR_STRING;
    HaveWins: Boolean;
    PrimaryWinsServer: IP_ADDR_STRING;
    SecondaryWinsServer: IP_ADDR_STRING;
    LeaseObtained: time_t;
    LeaseExpires: time_t;
  end;

  PIP_PER_ADAPTER_INFO = ^IP_PER_ADAPTER_INFO;
  IP_PER_ADAPTER_INFO = packed record
    AutoConfigEnabled: UINT;
    AutoConfigActive: UINT;
    CurrentDnsServer: PIP_ADDR_STRING;
    DnsServerList: IP_ADDR_STRING;
  end;

function GetAdaptersInfo(const pAdapterInfo: PIP_ADAPTER_INFO; const pOutBufLen: PULONG): DWORD; stdcall; external 'iphlpapi.dll';
function GetPerAdapterInfo(const pIfIndex: ULONG; const pAdapterInfo: PIP_PER_ADAPTER_INFO; const pOutBufLen: PULONG): DWORD; stdcall; external 'iphlpapi.dll';

function GetAdapterInfos: TAdapterInfosA;
function GetMacAddress(IpAddress: String): String;
function GetBroadcast(IpAddress, NetMask: String): String;

implementation

function GetAdapterInfos: TAdapterInfosA;
var
  pAdapterInfo: PIP_ADAPTER_INFO;
  AdapterInfo: IP_ADAPTER_INFO;
  BufLen: DWORD;
  Status: DWORD;
  strMAC: String;
  i: Integer;
  IndexA: Integer;
begin
  IndexA := 0;

  // Adapter IP Infos
  BufLen := SizeOf(AdapterInfo);
  pAdapterInfo := @AdapterInfo;
  Status := GetAdaptersInfo(nil, @BufLen);
  pAdapterInfo := AllocMem(BufLen);

  try
    // IP Infos
    Status := GetAdaptersInfo(pAdapterInfo, @BufLen);
    if (Status <> ERROR_SUCCESS) then
    begin
      Dispose(pAdapterInfo);
      Exit;
    end;

    while (pAdapterInfo <> nil) do
    begin
      SetLength(Result, IndexA + 1);

      // MAC Address
      strMAC := '';
      for i := 0 to pAdapterInfo^.AddressLength - 1 do
      begin
        {$RANGECHECKS OFF}
        strMAC := strMAC + '-' + IntToHex(pAdapterInfo^.Address[i], 2);
      end;
      Delete(strMAC, 1, 1);

      //
      Result[IndexA].Description := pAdapterInfo^.Description;
      Result[IndexA].IpAddress   := pAdapterInfo^.IpAddressList.IpAddress.ToString;
      Result[IndexA].MacAddress  := strMAC;
      Result[IndexA].SubNetMask  := pAdapterInfo^.IpAddressList.IpMask.ToString;
      Result[IndexA].Gateway     := pAdapterInfo^.GatewayList.IpAddress.ToString;
      Result[IndexA].DHCPServer  := pAdapterInfo^.DhcpServer.IpAddress.ToString;
      Result[IndexA].Broadcast   := GetBroadcast(
                                      pAdapterInfo^.IpAddressList.IpAddress.ToString,
                                      pAdapterInfo^.IpAddressList.IpMask.ToString
                                      );
      Result[IndexA].DNSServer   := 'not implemented!';

      pAdapterInfo := pAdapterInfo^.Next;
      Inc(IndexA);
    end;

  finally
    Dispose(pAdapterInfo);
  end;

end;

function GetMacAddress(IpAddress: String): String;
var
  i: Integer;
  InfoArr: TAdapterInfosA;
begin
  for i := 0 to High(InfoArr) do
  begin
    if (InfoArr[i].IpAddress = IpAddress) then
    begin
      Result := InfoArr[i].MacAddress;
    end;
  end;
end;

function GetBroadcast(IpAddress, NetMask: String): String;
var
  tmpIP, tmpNM: Integer;
begin
  tmpIP := StrToIP(IpAddress);
  tmpNM := StrToIP(NetMask);
  Result := IpToStr(tmpIP or (not tmpNM));
end;

end.

